#!/usr/bin/env python

# nightlight - A Blinkt-based night light controller
# By Andrew Wedgbury <wedge@sconemad.com>

import colorsys
import time
import datetime
import RPi.GPIO as GPIO
from blinkt import set_clear_on_exit, set_brightness, set_pixel, set_all, show

# Brightness in 'wakeup' mode
bright = 1.0

# Brightness and colour in 'dim' mode (orange by default)
dim = 0.7
dim_r = 255
dim_g = 8
dim_b = 0

# Hour and minute of the day when wakeup should activate
wake_hour = 6
wake_min = 30


spacing = 360.0 / 16.0
hue = 0
mode = 0 # 0:off 1:dim 2:wakeup
waketime = 0

BUTTON = 18 
GPIO.setmode(GPIO.BCM);
GPIO.setup(BUTTON, GPIO.IN, pull_up_down=GPIO.PUD_UP)

set_clear_on_exit()

def getWakeTime():
    tomorrow = 0
    t = time.time()
    ts = time.localtime(t)
    if (ts.tm_hour > wake_hour or
        (ts.tm_hour == wake_hour and ts.tm_min >= wake_min)):
        tomorrow = 1 
    t = time.mktime((ts.tm_year, ts.tm_mon, ts.tm_mday,
                     wake_hour, wake_min, 0,
                     ts.tm_wday, ts.tm_yday, ts.tm_isdst))
    if (tomorrow): t = t + 60 * 60 * 24
    return t
        
def reset():
    global mode
    global waketime
    mode = 1
    waketime = getWakeTime()
    print time.strftime("%c",time.localtime(waketime))
    
def button(channel):
    global mode
    mode = mode + 1
    if (mode > 1): mode = 0
    if (mode == 1): reset()

GPIO.add_event_detect(BUTTON, GPIO.BOTH, callback=button, bouncetime=300)

reset()

while True:

    if (mode == 0):
        set_all(0,0,0,0)
        show()
        time.sleep(1)
        continue
    
    if (mode == 1):
        if (time.time() > waketime):
            mode = 2
            continue
        set_all(dim_r,dim_g,dim_b,dim)
        show()
        time.sleep(1)
        continue

    set_brightness(bright)
    hue = int(time.time() * 100) % 360
    for x in range(8):
        offset = x * spacing
        h = ((hue + offset) % 360) / 360.0
        r, g, b = [int(c*255) for c in colorsys.hsv_to_rgb(h, 1.0, 1.0)]
        set_pixel(x,r,g,b)
    show()
    time.sleep(0.001)
